#!/usr/bin/env python
# -*- coding: utf-8 -*- 
#
#       Gissy - script for getting weather forecast from gismeteo.ru
#       
#       Copyright 2009 Dmitry Chirskov <freemindman@yandex.ru>
#
#		Gissy is free software; you can redistribute it and/or modify
#		it under the terms of the GNU General Public License as published by
#		the Free Software Foundation; either version 3 of the License, or
#		(at your option) any later version.
#
#		Gissy is distributed in the hope that it will be useful,
#		but WITHOUT ANY WARRANTY; without even the implied warranty of
#		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#		GNU General Public License for more details.
#
#		You should have received a copy of the GNU General Public License
#		along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os, sys, pwd, stat, getopt
import xml.sax, xml.sax.handler
import socket, urllib
import time

# Update every 3 hours
STALE_TIME = 600

class Gissy:
	def __init__(self):
		socket.setdefaulttimeout(1)
		town_code, day = None, None
		params = ("town", "day", "month", "year", "tod", "phenomena", 
					"temp_avr", "temp_max", "temp_min","wind_avr",
					"wind_max", "wind_min", "wind_dir", "pressure_avr",
					"pressure_max", "pressure_min", "relwet_avr", "relwet_max",
					"relwet_min", "comfort_avr", "comfort_max","comfort_min")
		
		try:
			opts, args = getopt.getopt(sys.argv[1:], "ht:d:", ["help", "town=", "day="])
		except getopt.GetoptError, err:
			print str(err)
			self.print_usage()
			sys.exit(2)
				
		for o, a in opts:
			if o in ("-h", "--help"):
				self.print_usage()
				sys.exit()
				
			elif o in ("-t", "--town"):
				try:
					town_code = int(a)
				except:
					print "Задан неверный код города!"
					sys.exit()
					
			elif o in ("-d", "--day"):
				try:
					day = int(a)
				except:
					print "Неверное смещение времени прогноза, должно быть от 0 до 3"
					sys.exit()
		
		if not town_code:
			print "Не задан код города!"
			sys.exit()
		
		if day < 0 or day > 3:
			print "Неверное смещение времени прогноза, должно быть от 0 до 3"
			sys.exit()
		
		param = sys.argv[len(sys.argv)-1] # get last argument
		if param in params:
			forecast = Forecast(town_code)
			if forecast.get():
				if param == "town":
					print forecast.town_name
				else:
					print forecast.map[day][param]
			else:
				print "-"
		else:
			print "Не задан параметр! Используйте 'gissy.py -h' для получения списка параметров"
			sys.exit()
				
	def print_usage(self):
		print ""
		print "Gissy - скрипт для получения прогноза погоды с gismeteo.ru"
		print "Рекомендуется использовать совместно с системным монитором conky (http://conky.sf.net)"
		print ""
		print "Запуск: gissy.py [ОПЦИИ] [ПАРАМЕТР]"
		print ""
		print "Опции :"
		print "  -h, --help			Выводит эту справку"
		print "  -t, --town			Код города"
		print "  -d, --day			Смещение времени прогноза, значение от 0 до 3"
		print ""
		print "Параметры :"
		print "  town				Название города"
		print "  day				День прогноза"
		print "  month				Месяц прогноза"
		print "  year				Год прогноза"
		print "  tod				Время суток"
		print "  phenomena			Выводит специальный символ для использования со шрифтом weather.ttf"
		print "  temp_avr			Средняя температура воздуха"
		print "  temp_min			Минимальная температура воздуха"
		print "  temp_max			Максимальная температура воздуха"
		print "  wind_avr			Средняя сила ветра"
		print "  wind_min			Минимальная сила ветра"
		print "  wind_max			Максимальная сила ветра"
		print "  wind_dir			Направление ветра"
		print "  pressure_avr		Среднее атмосферное давление"
		print "  pressure_min		Минимальное атомосферное давление"
		print "  pressure_max		Максимальное атмосферное давление"
		print "  relwet_avr			Средняя влажность воздуха"
		print "  relwet_min			Минимальная влажность воздуха"
		print "  relwet_max			Максимальная влажность воздуха"
		print "  comfort_avr		Среднее значение комфорта"
		print "  comfort_min		Минимальное значение комфорта"
		print "  comfort_max		Максимальное значение комфорта"

class Forecast(xml.sax.handler.ContentHandler):
	def __init__(self, town_code):
		self.map = []
		self.town_name = None
		self.town_code = town_code
		self.temp_file = "/tmp/gissy." + pwd.getpwuid(os.getuid())[0] + ".tmp"
		
	def startElement(self, name, attributes):
		if name == "TOWN":
			self.town_name = attributes["sname"]
		if name == "FORECAST":
			self.day = attributes["day"]
			self.month = attributes["month"]
			self.year = attributes["year"]
			self.tod = attributes["tod"]
		if name == "PHENOMENA":
			self.cloudiness = attributes["cloudiness"]
			self.precipitation = attributes["precipitation"]
		if name == "TEMPERATURE":
			self.temp_max = attributes["max"]
			self.temp_min = attributes["min"]
		if name == "WIND":
			self.windspd_max = attributes["max"]
			self.windspd_min = attributes["min"]
			self.wind_direction = attributes["direction"]
		if name == "PRESSURE":
			self.pressure_max = attributes["max"]
			self.pressure_min = attributes["min"]
		if name == "RELWET":
			self.relwet_max = attributes["max"]
			self.relwet_min = attributes["min"]
		if name == "HEAT":
			self.comfort_max = attributes["max"]
			self.comfort_min = attributes["min"]
			
	def endElement(self, name):
		if name == "TOWN":
			self.town_name = urllib.unquote(self.town_name.encode("cp1251")).decode("cp1251")
		if name == "FORECAST":
			self.map.append (
			{"day":				self.day,
			 "month": 			self.month,
			 "year": 			self.year,
			 "tod":				self.get_tod_string(int(self.tod)),
			 "phenomena":		self.get_phenomena(int(self.precipitation), int(self.cloudiness)),
			 
			 "temp_avr":		self.get_polarity((int(self.temp_max) + int(self.temp_min))/2),
			 "temp_max":		self.temp_max,
			 "temp_min":		self.temp_min,
			 
			 "wind_avr":		(int(self.windspd_max) + int(self.windspd_min))/2,
			 "wind_max":		self.windspd_max,
			 "wind_min":		self.windspd_min,
			 "wind_dir":		self.get_wind_string(int(self.wind_direction)),
			 
			 "pressure_avr":	(int(self.pressure_max) + int(self.pressure_min))/2,
			 "pressure_max":	self.pressure_max,
			 "pressure_min":	self.pressure_min,
			 
			 "relwet_avr":		(int(self.relwet_max) + int(self.relwet_min))/2,
			 "relwet_max":		self.relwet_max,
			 "relwet_min":		self.relwet_min,
			 
			 "comfort_avr":		self.get_polarity((int(self.comfort_max) + int(self.comfort_min))/2),
			 "comfort_max":		self.comfort_max,
			 "comfort_min":		self.comfort_min
			})
	
	def get_tod_string(self, tod):
		""" Convert time of day code to string """
		if tod == 0:
			return "Ночь"
		elif tod == 1:
			return "Утро"
		elif tod == 2:
			return "День"
		elif tod == 3:
			return "Вечер"
	
	def get_wind_string(self, direction):
		""" Convert wind code to string """
		if direction == 0:
			return "С"
		elif direction == 1:
			return "С-В"
		elif direction == 2:
			return "В"
		elif direction == 3:
			return "Ю-В"
		elif direction == 4:
			return "Ю"
		elif direction == 5:
			return "Ю-З"
		elif direction == 6:
			return "З"
		elif direction == 7:
			return "С-З"
			
	def get_polarity(self, integer):
		if integer > 0:
			return "+" + str(integer)
		else:
			return str(integer)
	
	def get_phenomena(self, precipitation, cloudiness):
		""" Returns special character for font weather.ttf """
		if precipitation == 4 or precipitation == 5:
			if cloudiness == 2:
				return "s"
			elif cloudiness == 3:
				return "t"
					
		elif precipitation == 6 or precipitation == 7:
			if cloudiness == 2:
				return "r"
			elif cloudiness == 3:
				return "o"
		
		elif precipitation == 8:
			return "m"
			
		elif precipitation == 10:
			if cloudiness == 0:
				return "a"
			elif cloudiness == 1:
				return "c"
			elif cloudiness == 2:
				return "d"
			elif cloudiness == 3:
				return "e"
		
		elif precipitation == 9:
			return "9"
		else:
			return "9"
	
	def fetch(self):
		""" Get XML document from server and save it to temp file """
		try:
			data = urllib.urlopen("http://informer.gismeteo.ru/xml/%s_1.xml"%(self.town_code))
			tmp = open(self.temp_file, "w")
			tmp.write(data.read())
			tmp.close()
		except:
			pass
					
	def load(self):
		if os.path.isfile(self.temp_file):
			if (int(time.time()) - int(os.stat(self.temp_file)[stat.ST_MTIME])) < STALE_TIME :
				# Temp file is fresh. Reading...
				return self.temp_file
			else:
				# Temp file is stale. Refreshing...
				self.fetch()
				return self.temp_file
		else:
			self.fetch()
			return self.temp_file
			
	def get(self):
		try:
			parser = xml.sax.make_parser()
			parser.setContentHandler(self)
			parser.parse(self.load())
			return True
		except:
			return False

if __name__ == "__main__":
	Gissy()
